/*
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * IBM Configure, Price , Quote (5725 D11)
 *
 * (c) Copyright IBM Corp. 2015, 2016 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets,
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */

var modelLoaded = false;

/*global $ */
/* jshint maxlen: 250 */
(function () {
    'use strict';

    angular.module('ibmConfiguratorApp.configurator').directive('ibmConfigurator', ibmConfigurator);

    ibmConfigurator.$inject = ['configuratorService', '$rootScope', '$sce', '$document', '$window'];
    /* @ngInject */
    function ibmConfigurator(configuratorService, $rootScope, $sce, $document, $window) {
        var directive = {
            restrict: 'AE',
            scope: {
                modelData: '=model',
                resize: '&resize',
            },
            link: link,
            template: '<ng-include ng-if="model" ' +
                'src="getTemplate(model.properties[propertyName].value)">' +
                '</ng-include>'
        };
        return directive;

        function link(scope, element, attrs) {
            configuratorService.initConfig();
            process();

            scope.setModelResponseData = function (response) {
                //console.log('setModelResponseData', response);
                scope.model.tabs = response.tabs;
                scope.model.options = response.options;
                scope.model.parentConfigurations = response.parentConfigurations;
                scope.model.modelproperties = response.properties;
                scope.model.modelHierarchy = response.modelHierarchy;
                scope.model.messages = response.messages;
                scope.model.properties = response.properties;
            };

            function generateBOM(picks, showAs) {
                var setBOMFocus = false;

                if (showAs === undefined || showAs === '') {
                    scope.showAs = 'table';
                    showAs = 'table';
                    setBOMFocus = true;
                } else {
                    scope.showAs = showAs;
                }

                configuratorService.generateBOM(picks, showAs).then(function (response) {
                    if (scope.showAs === 'table') {
                        scope.BOM = response.data;
                        scope.BOMTable = [];
                        pushToBOMTable(response.data.lineItem, 0);
                        console.log(scope.BOMTable);
                    } else {
                        scope.BOM = response.data;
                    }

                    if (setBOMFocus) {
                        $('#model_header').attr('tabindex', '-1').css('outline', 'none');
                        $('#model_header').focus();
                    }
                });

                function pushToBOMTable(data, level) {
                    for (var i in data) {
                        if (data.hasOwnProperty(i)) {
                            var line = {};
                            var space = '';
                            for (var count = 0; count < level; count++) {
                                space += '&nbsp;&nbsp;&nbsp;';
                            }
                            line.name = data[i].name;
                            line.value = data[i].uevValue;
                            line.desc = $sce.trustAsHtml(space + data[i].description);
                            line.quantity = data[i].quantity;
                            line.price = data[i].price;
                            scope.BOMTable.push(line);
                            if (data[i].lineItem !== null && data[i].lineItem !== undefined) {
                                pushToBOMTable(data[i].lineItem, level + 1);
                            }
                        }
                    }
                }
            }

            function updateCurrentTab(index) {
                $rootScope.currentTab = index;
            }

            function process(response) {
                scope.bomXml = $window.bomXml;
                if (scope.bomXml !== null && scope.bomXml !== '' && typeof (scope.bomXml) !== 'undefined') {
                    reconfigureFromXML(scope.bomXml);
                } else {
                    $rootScope.showLoadingMask = true;
                    configuratorService.processConfigurationPicks(scope.modelData)
                        .then(success)
                        .catch(fail);
                    scope.updateCurrentTab = updateCurrentTab;
                    scope.generateBOM = generateBOM;
                    scope.reconfigure = reconfigure;
                    scope.returnfromSubmodel = returnfromSubmodel;

                    scope.total = 0;
                    $rootScope.currentTab = 0;
                    scope.messageCount = 0;
                    scope.showBOM = false;
                    //	            scope.showAs='json';
                }
            }

            function success(response) {
                scope.model = response.data;
                $rootScope.optArray = [];
                $rootScope.picks = scope.model.picks;
                $rootScope.inputProperties = scope.model.inputProperties;
                $rootScope.unproceeedPropertyPool = scope.model.propertyPool;
                $rootScope.propertyPool = preProcessPropertyPool(scope.model.propertyPool);
                $rootScope.summary = scope.model.summary;
                $rootScope.total = scope.model.properties._totalPrice.value;
                $rootScope.traceLog = scope.model.traceLog;
                scope.parentConfigurations = scope.model.parentConfigurations;
                //scope.model.parentconfiguration = [];
                scope.propertyName = 'UI: JSP FILENAME';

                if (typeof (scope.model.properties._modelVersion) !== 'undefined' && scope.model.properties._modelVersion !== null) {
                    if (typeof (scope.model.properties._modelVersion.value) !== 'undefined' && scope.model.properties._modelVersion.value !== null) {
                        scope.modelType = scope.model.properties._modelVersion.value; // 'Omni';
                        if (scope.modelType === 'v2') {
                            scope.propertyName = 'UI: TEMPLATE NAME';
                        } else {
                            scope.propertyName = 'UI: JSP FILENAME';
                        }
                    }
                }

                if (scope.model.properties['_expandedDescription'] !== undefined) {
                    $rootScope.modeldisname = scope.model.properties['_expandedDescription'].value;
                } else {
                    // old code - without expand/gather
                    if (scope.model.properties['UI: DISPLAY NAME'] !== undefined) {
                        if ('' !== scope.model.properties['UI: DISPLAY NAME'].value) {
                            $rootScope.modeldisname = scope.model.properties['UI: DISPLAY NAME'].value;
                        }
                    } else {
                        $rootScope.modeldisname = scope.model.properties['_description'].value;
                    }
                }

                if (scope.model.properties['UI: AUTOMATIC POST'] !== undefined) {
                    $rootScope.autoUpdate = scope.model.properties['UI: AUTOMATIC POST'].value;
                } else {
                    $rootScope.autoUpdate = 'UPDATE';
                }

                scope.messageCount = 0;
                if (typeof (scope.model.messages) !== 'undefined' && scope.model.messages !== null) {
                    scope.messageCount = scope.model.messages.length;
                }
                //scope.$broadcast('modelLoaded');
                $rootScope.showLoadingMask = false;
                modelLoaded = true;
            }

            function fail(error) {
                var msg = 'query for model failed. ' + error.data.description;
                console.error(msg);
                return msg;
            }

            scope.$on('modelLoaded', resize);

            function resize() {
                $(window).on('resize', scope.resize).trigger('resize'); // on
                // page
                // load
            }

            function reconfigureFromXML(bomXml) {
                configuratorService.extractPicksFromConfigurationBOM(bomXml, 'xml').success(function (response) {
                    $rootScope.modelcountry = response.country;
                    $rootScope.currency = $rootScope.currency;
                    $rootScope.country = response.country;
                    $rootScope.modellanguage = response.language;
                    $rootScope.modelPath = response.path;
                    $rootScope.modelDepth = 0;
                    $rootScope.picks = response.picks;
                    $rootScope.inputProperties = response.inputProperties;
                    $window.bomXml = '';
                    process(response);
                    $rootScope.showBOM = false;
                });
            }

            function reconfigure() {
                var bom = JSON.parse(document.getElementById('bom_text_area').value);
                bom.currency = $rootScope.currency;
                bom.country = $rootScope.modelcountry;
                bom.language = $rootScope.modellanguage;
                configuratorService.extractPicksFromConfigurationBOM(bom).success(function (response) {
                    $rootScope.modelcountry = response.country;
                    $rootScope.currency = $rootScope.currency;
                    $rootScope.country = response.country;
                    $rootScope.modellanguage = response.language;
                    $rootScope.modelPath = response.path;
                    $rootScope.modelDepth = 0;
                    $rootScope.picks = response.picks;
                    $rootScope.inputProperties = response.inputProperties;
                    process(response);
                    $rootScope.showBOM = false;
                });
            }

            function returnfromSubmodel(picks, modelDepth, abandon) {
                configuratorService.returnFromSubModel($rootScope.picks, scope.model.parentConfigurations, $rootScope.modelDepth,
                    scope.model.nestedConfiguration, abandon).success(function (response) {
                    $rootScope.picks = response.picks;
                    $rootScope.inputProperties = response.inputProperties;
                    // scope.model.summary = response.summary;
                    scope.model.tabs = response.tabs;
                    scope.model.options = response.options;
                    scope.model.parentConfigurations = response.parentConfigurations;
                    scope.model.properties = response.properties;
                    scope.model.path = response.path;
                    scope.model.nestedConfiguration = response.nestedConfiguration;
                    scope.model.messages = response.messages;
                    $rootScope.modelPath = scope.model.path;
                    $rootScope.modelDepth = response.modelDepth;
                    scope.model.modelHierarchy = response.modelHierarchy;
                    $rootScope.unproceeedPropertyPool = response.propertyPool;
                    $rootScope.propertyPool = preProcessPropertyPool(response.propertyPool);
                    $rootScope.summary = response.summary;
                    $rootScope.total = response.properties._totalPrice.value;
                    $rootScope.currentTab = $rootScope.passcurrentTab;
                    $rootScope.traceLog = response.traceLog;
                    
                    if (response.properties['UI: DISPLAY NAME'] !== undefined) {
                        if ('' !== response.properties['UI: DISPLAY NAME'].value) {
                            $rootScope.modeldisname = response.properties['UI: DISPLAY NAME'].value;
                        }
                    } else {
                        $rootScope.modeldisname = response.properties['_description'].value;
                    }

                    if (scope.model.properties['UI: AUTOMATIC POST'] !== undefined) {
                        $rootScope.autoUpdate = scope.model.properties['UI: AUTOMATIC POST'].value;
                    } else {
                        $rootScope.autoUpdate = 'UPDATE';
                    }
                }).catch(fail);
            }

            scope.getTemplate = getTemplate;

            function getTemplate(templateName) {
                if (templateName === undefined) {
                    if (scope.modelType === 'v2') {
                        templateName = 'AccordionTemplate';
                    } else {
                        templateName = 'configurator.jsp';
                    }
                }
                scope.template = configuratorService.resolveTemplate(templateName, scope.modelType);
                if (templateName === 'NormalTemplate' || templateName === 'configurator.jsp') {
                    $document.ready(function () {
                        $('.title_info').css('borderBottom', '2px solid #58C0C0');
                        $('.title_info').css('margin', '5px 0');
                        $('.title_info').css('padding', '10px 0 5px');
                    });
                }
                return scope.template.file;
            }
            function preProcessPropertyPool(propertyPool) {
                var newPropertyPool = [];
                for (var i in propertyPool) {
                    if (propertyPool.hasOwnProperty(i)) {
                        for (var j in propertyPool[i]) {
                            if (propertyPool[i].hasOwnProperty(j)) {
                                newPropertyPool.push(propertyPool[i][j]);
                            }
                        }
                    }
                }
                return newPropertyPool;
            }

            scope.calculateQuanttity = function(item) {
                if(typeof $rootScope.unproceeedPropertyPool[item.fullName] !== "undefined" && typeof $rootScope.unproceeedPropertyPool[item.fullName].kitQty !== "undefined" && parseInt($rootScope.unproceeedPropertyPool[item.fullName].kitQty.value)) {
                    return parseInt($rootScope.unproceeedPropertyPool[item.fullName].kitQty.value)*item.quantity;
                } else {
                    return item.quantity;
                }
            }

            
        }
    }
})();
