/*
 * IBM Confidential 
 *
 * OCO Source Materials
 *
 * IBM Configure, Price , Quote (5725 D11)
 *
 * (c) Copyright IBM Corp. 2015, 2016 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets,
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */

/*jshint maxlen: 250 */
/*global angular */
(function () {
    'use strict';

    angular.module('ibmConfiguratorApp.configurator').factory('configuratorService', configuratorService);

    configuratorService.$inject = ['$http', '$q', 'logger', '$rootScope', 'configuratorConfig', '$window', '$location'];
    /* @ngInject */
    function configuratorService($http, $q, logger, $rootScope, configuratorConfig, $window, $location) {
        var excludeFilter = [
		// { 'class': 'TraceLog'},
		// {'class': 'PropertyPool'},
		// {'class': 'Property', 'name': 'UI: MODEL READONLY'},
		// {'class': 'Property', 'name': '_cacheKey'},
		// {'class': 'Property', 'name': '_cacheKey'},
		// {'class': 'Property', 'name': '_lastModifiedDate'},
		// {'class': 'Property', 'name': '_itemKey'},
		// {'class': 'Property', 'name': '_fullName'},
		// {'class': 'Property', 'name': '_modelTabs'},
		// {'class': 'Property', 'name': '_modelName'},
		// {'class': 'Property', 'name': 'UI: PRODUCT NAME'},
		// {'class': 'Property', 'name': '_sequence'},
		// {'class': 'Property', 'name': '_endDate'},
		// {'class': 'Property', 'name': '_sku'},
		// {'class': 'Property', 'name': '_constraintCount'},
		// {'class': 'Property', 'name': '_rulePick'},
		// {'class': 'Property', 'name': '_ratio'},
		// {'class': 'Property', 'name': 'UI: AUTOMATIC POST'},
		// {'class': 'Property', 'name': '_enddate'},
		// {'class': 'Property', 'name': 'UI: PRODUCT UOM'},
		// {'class': 'Property', 'name': '_constraintType'},
		// {'class': 'Property', 'name': '_startDate'},
		// {'class': 'Property', 'name': 'UI: PRODUCT ID'},
		// {'class': 'Property', 'name': '_pickItems'},
		// {'class': 'Property', 'name': '_startdate'},
		// {'class': 'Property', 'name': '_picks'},
		// {'class': 'Property', 'name': '_fileSize'},
		// {'class': 'Property', 'name': '_errorsInModel'},
		// {'class': 'Property', 'name': 'UI: READONLY'},
		// {'class': 'Property', 'name': '_isConstrained'},
		// {'class': 'Property', 'name': 'UI: PRICING SKU'},
		// {'class': 'Property', 'name': 'UI: REQUIRED'},
		// {'class': 'Property', 'name': 'UI: END ROW'},
		// {'class': 'Property', 'name': 'UI: PRODUCT DESCRIPTION'},
		// {'class': 'Property', 'name': '_name'},
		// {'class': 'Property', 'name': '_rawRatio'},
		// {'class': 'Property', 'name': '_tabLevel'},
		// {'class': 'Property', 'name': '_constraintMessage'},
		// {'class': 'Property', 'name': '_pickOverride'},
		// {'class': 'Property', 'name': 'UI: PREVENT SELECTION'},
		// {'class': 'Property', 'name': 'UI: IGNORE IN QUOTE'},
		// {'class': 'Property', 'name': 'UI: SHOW ITEM IMAGES'},
		// {'class': 'Property', 'name': '_rendered'},
		// {'class': 'Property', 'name': 'UI: SUPPRESS NAME DISPLAY'},
		// {'class': 'Property', 'name': 'UI: ITEM IMAGE NAME'},
		// {'class': 'Property', 'name': '_isPickRemoved'}
		];
		
		function setExcludeForDebug(isDebugEnabled) {
			if(isDebugEnabled) {
				excludeFilter = [];
			} else {
				excludeFilter = [{ 'class': 'TraceLog'}];
			}
		};
		
		$rootScope.$watch(function() { return $rootScope.debugEnabled; },
			function(newValue, oldValue) {
				setExcludeForDebug(newValue);
			}
		);
			 
        var service = {
            getModel: getModel,
            initConfig: initConfig,
            processConfigurationPicks: processConfigurationPicks,
            resolveTemplate: resolveTemplate,
            addConfigurationPicks: addConfigurationPicks,
            removeConfigurationPicks: removeConfigurationPicks,
            removeAndAddConfigurationPicks: removeAndAddConfigurationPicks,
            getModelDetails: getModelDetails,
            getModelCache: getModelCache,
			clearModelCache: clearModelCache,
			clearModelCacheEntries: clearModelCacheEntries,
            generateBOM: generateBOM,
            punchIntoSubModel: punchIntoSubModel,
            instantiateSubModel: instantiateSubModel,
            extractPicksFromConfigurationBOM: extractPicksFromConfigurationBOM,
            removeSubModel: removeSubModel,
            copySubModel: copySubModel,
            returnFromSubModel: returnFromSubModel,
            resolveConstrainedItem: resolveConstrainedItem,
            trapTestcase: trapTestcase,
            findTestcase: findTestcase,
            executeTestcase: executeTestcase,
            deleteTestcase: deleteTestcase,
            rebase: rebase,
            generalRegressionRequest: generalRegressionRequest
        };

        return service;

        function getModel(modelData) {
            // return $http.get('json/response.json');
        }

        function initConfig() {
            	
            if (typeof ($window.configuratorServer) === 'undefined') {
                $rootScope.configuratorServer = window.location.protocol + '//' + window.location.hostname;
                if (window.location.port !== 80) {
                    $rootScope.configuratorServer += ':' + window.location.port + '/configurator';
                }
            } else {
                $rootScope.configuratorServer = $window.configuratorServer;
            }
            if (typeof ($window.vmServer) === 'undefined') {
                $rootScope.vmServer = window.location.protocol + '//' + window.location.hostname;
                if (window.location.port !== 80) {
                    $rootScope.vmServer += ':' + window.location.port + '/VisualModeler';
                }
            } else {
                $rootScope.vmServer = $window.vmServer;
            }
			
			// The value of this property determines whether TraceLog should be returned or not.
			var currentParams = $location.search();
			$rootScope.debugEnabled=currentParams.DEBUG;
			setExcludeForDebug($rootScope.debugEnabled);
        }

        function extractPicksFromConfigurationBOM(bom, contentType) {
            console.log('extractPicksFromConfigurationBOM............');
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/bom/configuration?responseFormat=json';
            console.log('url=' + url);

            if (typeof (contentType) !== 'undefined' && contentType === 'xml') {
                contentType = 'application/xml; charset=UTF-8';
            } else {
                contentType = 'application/json; charset=UTF-8';
            }
            return $http.put(url, bom, {
                headers: {
                    'Content-Type': contentType
                }
            });
        }

        function processConfigurationPicks() {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            if (typeof ($rootScope.modelDepth) === 'undefined' || $rootScope.modelDepth === null) {
                $rootScope.modelDepth = '0';
            }

            if (typeof ($rootScope.picks) !== 'undefined' || $rootScope.picks != null) {
                return $http.put(url, {

                    'action': 'PROCESS',
                    'country': $rootScope.modelcountry,
                    'currency': $rootScope.currency,
                    'depth': '',
                    'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                    'inputProperties': $rootScope.inputProperties,
                    'customerID': $rootScope.customerID,
                    'language': $rootScope.modellanguage,
                    'path': $rootScope.modelPath,
                    'picks': $rootScope.picks,
                    'modelDepth': $rootScope.modelDepth,
                    'firingPhase': $rootScope.firingPhase,
                    'clientProperties': $rootScope.clientProperties,
                    'exclude': excludeFilter
                });
            } else {
                return $http.put(url, {

                    'action': 'PROCESS',
                    'country': $rootScope.modelcountry,
                    'currency': $rootScope.currency,
                    'depth': '',
                    'customerID': $rootScope.customerID,
                    'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                    'inputProperties': $rootScope.inputProperties,
                    'language': $rootScope.modellanguage,
                    'path': $rootScope.modelPath,
                    'modelDepth': $rootScope.modelDepth,
                    'firingPhase': $rootScope.firingPhase,
                    'clientProperties': $rootScope.clientProperties,
                    'exclude': excludeFilter
                });
            }
        }

        function instantiateSubModel(option, picks, parentconfiguration, nestedConfiguration) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            if (typeof ($rootScope.modelDepth) === 'undefined' || $rootScope.modelDepth === null) {
                $rootScope.modelDepth = '0';
            }
            return $http.put(url, {

                'action': 'INSTANTIATE_SUB_MODEL',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'modelDepth': $rootScope.modelDepth,
                'parentConfigurations': parentconfiguration,
                'nestedConfiguration': nestedConfiguration,
                'inputProperties': $rootScope.inputProperties,
                'picks': $rootScope.picks,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'exclude': excludeFilter,
                'addSubModel': [{
                    'item': option.path,
                    'quantity': 1,
                    'value': ''
				}]
            });
        }

        function removeSubModel(option, picks, parentconfiguration, nestedConfiguration) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            if (typeof ($rootScope.modelDepth) === 'undefined' || $rootScope.modelDepth === null) {
                $rootScope.modelDepth = '0';
            }
            return $http.put(url, {

                'action': 'INSTANTIATE_SUB_MODEL',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'modelDepth': $rootScope.modelDepth,
                'picks': $rootScope.picks,
                'parentConfigurations': parentconfiguration,
                'nestedConfiguration': nestedConfiguration,
                'inputProperties': $rootScope.inputProperties,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'exclude': excludeFilter,
                'removeSubModel': [{
                    'item': option.path,
                    'quantity': 1,
                    'value': ''
				}]
            });
        }

        function copySubModel(option, picks, parentconfiguration, nestedConfiguration) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            if (typeof ($rootScope.modelDepth) === 'undefined' || $rootScope.modelDepth === null) {
                $rootScope.modelDepth = '0';
            }
            return $http.put(url, {

                'action': 'INSTANTIATE_SUB_MODEL',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'modelDepth': $rootScope.modelDepth,
                'picks': $rootScope.picks,
                'parentConfigurations': parentconfiguration,
                'nestedConfiguration': nestedConfiguration,
                'inputProperties': $rootScope.inputProperties,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'exclude': excludeFilter,
                'copySubModel': [{
                    'item': option.path,
                    'quantity': 1,
                    'value': ''
				}]
            });
        }

        function findTestcase(testId, groupId) {
            // just for test, will change to rootscope configuration
            var url = $rootScope.vmServer + '/api/organization/' + $rootScope.organizationCode + '/regression?responseFormat=json';
            if (testId === undefined) {
                testId = '';
            }

            if (groupId === undefined) {
                groupId = '';
            }

            return $http.post(url, {
                'testId': testId,
                'groupId': groupId,
                'operation': 'FIND'
            });
        }

        function executeTestcase(testId, groupId, testKey) {
            // just for test, will change to rootscope configuration

            var url = $rootScope.vmServer + '/api/organization/' + $rootScope.organizationCode + '/regression?responseFormat=json';

            return $http.post(url, {
                'testId': testId,
                'groupId': groupId,
                'testKey': testKey,
                'operation': 'EXECUTE'
            });
        }

        function deleteTestcase(testId, groupId, testKey) {
            // just for test, will change to rootscope configuration

            var url = $rootScope.vmServer + '/api/organization/' + $rootScope.organizationCode + '/regression?responseFormat=json';

            return $http.post(url, {
                'testId': testId,
                'groupId': groupId,
                'testKey': testKey,
                'operation': 'DELETE'
            });
        }

        function rebase(testId, groupId, testKey) {
            // just for test, will change to rootscope configuration

            var url = $rootScope.vmServer + '/api/organization/' + $rootScope.organizationCode + '/regression?responseFormat=json';

            return $http.post(url, {
                'testId': testId,
                'groupId': groupId,
                'testKey': testKey,
                'operation': 'REBASE'
            });
        }

        function generalRegressionRequest(method, format, testId, groupId, testKey) {
            var url = $rootScope.vmServer + '/api/organization/' + $rootScope.organizationCode + '/regression?responseFormat=' + format.toLowerCase();

            return $http.post(url, {
                'testId': testId,
                'groupId': groupId,
                'testKey': testKey,
                'operation': method
            });
        }

        function trapTestcase(testId, groupId, description) {
            // just for test, will change to rootscope configuration
            var url = $rootScope.vmServer + '/api/organization/' + $rootScope.organizationCode + '/regression?responseFormat=json';

            if (typeof ($rootScope.modelDepth) === 'undefined' || $rootScope.modelDepth === null) {
                $rootScope.modelDepth = '0';
            }

            if (typeof ($rootScope.picks) !== 'undefined' || $rootScope.picks !== null) {
                return $http.post(url, {
                    'testId': testId,
                    'groupId': groupId,
                    'description': description,
                    'operation': 'TRAP',
                    'managePicks': {
                        'action': 'PROCESS',
                        'country': $rootScope.modelcountry,
                        'currency': $rootScope.currency,
                        'depth': '',
                        'customerID': $rootScope.customerID,
                        'inputProperties': $rootScope.inputProperties,
                        'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                        'language': $rootScope.modellanguage,
                        'path': $rootScope.modelPath,
                        'picks': $rootScope.picks,
                        'modelDepth': $rootScope.modelDepth,
                        'exclude': excludeFilter
                    }
                });
            } else {
                return $http.post(url, {
                    'testId': testId,
                    'groupId': groupId,
                    'description': description,
                    'operation': 'TRAP',
                    'managePicks': {
                        'action': 'PROCESS',
                        'country': $rootScope.modelcountry,
                        'currency': $rootScope.currency,
                        'depth': '',
                        'customerID': $rootScope.customerID,
                        'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                        'language': $rootScope.modellanguage,
                        'path': $rootScope.modelPath,
                        'inputProperties': $rootScope.inputProperties,
                        'modelDepth': $rootScope.modelDepth,
                        'exclude': excludeFilter
                    }
                });
            }
        }

        function getParent(opt) {
            var n = opt.path.lastIndexOf('.');
            return opt.path.substring(0, n);
        }

        function pathInPicks(picks, path) {
            var flag = false;
            if (picks) {
                for (var i=0; i<picks.length; i++) {
                    if (path === picks[i].item) {
                        flag = true;
                        break;
                    }
                }
            }   
            return flag;
        }

        // Validates that the input string is a valid date formatted as
        // 'mm/dd/yyyy'
        function isValidDate(dateString) {
            // First check for the pattern
            if (!/^\d{1,2}\/\d{1,2}\/\d{4}$/.test(dateString)) {
                return false;
            }

            // Parse the date parts to integers
            var parts = dateString.split('/');
            var day = parseInt(parts[1], 10);
            var month = parseInt(parts[0], 10);
            var year = parseInt(parts[2], 10);

            // Check the ranges of month and year
            if (year < 1000 || year > 3000 || month === 0 || month > 12) {
                return false;
            }

            var monthLength = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
            // Adjust for leap years
            if (year % 400 === 0 || (year % 100 !== 0 && year % 4 === 0)) {
                monthLength[1] = 29;
            }

            // Check the range of the day
            return day > 0 && day <= monthLength[month - 1];
        }

        function getMultiAddPicksInput(options, picks) {
            var jsonArr = [];

            if (options instanceof Array) {
                for (var i=0; i<options.length; i++) {
                    jsonArr = jsonArr.concat(getAddPicksInput(options[i], picks));
                }
            } else {
                jsonArr = getAddPicksInput(options, picks);
            }

            return jsonArr;
        }

        function getAddPicksInput(opt, picks) {
            var parentPath = getParent(opt);
            if (opt.quantity === undefined) {
                opt.quantity = 1;
            }
            if (opt.value === undefined) {
                opt.value = '';
            }

            if (opt.uEVSelection !== undefined) {
                opt.value = opt.uEVSelection;
            }

            if (opt.properties !== undefined && opt.properties['UI: UEV TYPE'] !== undefined && opt.properties['UI: UEV TYPE'].value === 'date') {
                opt.value = new Date(opt.uEVSelection).getTime();
            }

            var jsonArr = [];
            if (pathInPicks(picks, parentPath)) {
                jsonArr.push({
                    'item': opt.path,
                    'quantity': opt.quantity,
                    'value': opt.value
                });
            } else {
                jsonArr.push({
                    'item': parentPath,
                    'quantity': 1,
                    'value': ''
                });
                jsonArr.push({
                    'item': opt.path,
                    'quantity': opt.quantity,
                    'value': opt.value
                });
            }
            return jsonArr;
        }

        function addConfigurationPicks(opt, picks, parentconfiguration,
            nestedConfiguration, modelDepth) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            return $http.put(url, {
                'action': 'PROCESS',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'picks': $rootScope.picks,
                'modelDepth': modelDepth,
                'parentConfigurations': parentconfiguration,
                'inputProperties': $rootScope.inputProperties,
                'nestedConfiguration': nestedConfiguration,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'exclude': excludeFilter,
                'addPick': getMultiAddPicksInput(opt, picks),
            });
        }

        function getPathToRemovePicks(options) {
            var jsonArr = [];
            if (options instanceof Array) {
                for (var i=0; i<options.length; i++) {
                    jsonArr.push({
                        item: options[i].path
                    });
                }
            } else {
                jsonArr.push({
                    item: options.path
                });
            }
            return jsonArr;
        }

        function removeConfigurationPicks(opt, picks, parentconfiguration,
            nestedConfiguration, modelDepth) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            return $http.put(url, {
                'action': 'PROCESS',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'picks': $rootScope.picks,
                'removePick': getPathToRemovePicks(opt),
                'modelDepth': modelDepth,
                'parentConfigurations': parentconfiguration,
                'inputProperties': $rootScope.inputProperties,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'exclude': excludeFilter,
                'nestedConfiguration': nestedConfiguration
            });
        }

        function removeAndAddConfigurationPicks(remOpt, addOpt, picks,
            parentconfiguration, nestedConfiguration, modelDepth) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            return $http.put(url, {
                'action': 'PROCESS',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'modelDepth': modelDepth,
                'parentConfigurations': parentconfiguration,
                'nestedConfiguration': nestedConfiguration,
                'inputProperties': $rootScope.inputProperties,
                'picks': $rootScope.picks,
                'removePick': getPathToRemovePicks(remOpt),
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'exclude': excludeFilter,
                'addPick': getMultiAddPicksInput(addOpt, picks)
            });
        }

        function resolveConstrainedItem(resolveItem, parentconfiguration,
            nestedConfiguration, modelDepth) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';
            return $http.put(url, {
                'resolveItem': resolveItem,
                'action': 'PROCESS',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'picks': $rootScope.picks,
                'modelDepth': modelDepth,
                'parentConfigurations': parentconfiguration,
                'nestedConfiguration': nestedConfiguration,
                'inputProperties': $rootScope.inputProperties,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'exclude': excludeFilter
            });
        }

        function generateBOM(picks, showAs) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration/bom?responseFormat=' + showAs;
            if (showAs === '') {
                showAs = 'json';
            }
            return $http.put(url, {
                'action': 'PROCESS',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'inputProperties': $rootScope.inputProperties,
                'path': $rootScope.modelPath,
                'modelDepth': $rootScope.modelDepth,
				'clientProperties': $rootScope.clientProperties,
                'picks': $rootScope.picks
            });
        }

        function getModelDetails() {
            return $http
                .get($rootScope.configuratorServer + '/api/organization/matrix/models?language=en&country=US&responseFormat=json');
        }

        function resolveTemplate(templateName, modelType) {
            //			if (modelType == 'v2') {
            //				if (typeof (templateMappingV2Ext) == 'undefined') {
            //					return templateMappingV2.templates[templateName];
            //				} else {
            //					return templateMappingV2Ext.templates[templateName];
            //				}
            //			} else {
            //				if (typeof (templateMappingExt) == 'undefined') {
            //					return templateMapping.templates[templateName];
            //				} else {
            //					return templateMappingExt.templates[templateName];
            //				}
            //			}

            if (modelType === 'v2') {
                return configuratorConfig.getTemplatesV2ByName(templateName);
            } else {
                return configuratorConfig.getTemplatesByName(templateName);
            }
        }

        function returnFromSubModel(picks, parentconfiguration, modelDepth,
            nestedConfiguration, abandon) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';

            return $http.put(url, {

                'action': 'RETURN_FROM_SUB_MODEL',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'modelDepth': modelDepth,
                'parentConfigurations': parentconfiguration,
                'nestedConfiguration': nestedConfiguration,
                'inputProperties': $rootScope.inputProperties,
                'abandon': abandon,
                'exclude': excludeFilter,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'picks': $rootScope.picks
            });
        }

        function punchIntoSubModel(picks, pucnchinitem, modelDepth,
            parentconfiguration) {
            var url = $rootScope.configuratorServer + '/api/organization/' + $rootScope.organizationCode + '/configuration?responseFormat=json';

            return $http.put(url, {

                'action': 'PUNCH_INTO_SUB_MODEL',
                'country': $rootScope.modelcountry,
                'currency': $rootScope.currency,
                'depth': '',
                'customerID': $rootScope.customerID,
                'ignoreCustomerEntitlements': $rootScope.ignoreCustomerEntitlements,
                'language': $rootScope.modellanguage,
                'path': $rootScope.modelPath,
                'modelDepth': modelDepth,
                'punchInItem': pucnchinitem,
                'parentConfigurations': parentconfiguration,
                'inputProperties': $rootScope.inputProperties,
                'exclude': excludeFilter,
                'firingPhase': $rootScope.firingPhase,
                'clientProperties': $rootScope.clientProperties,
                'picks': $rootScope.picks
            });
        }
        
        function getModelCache() {			
            var url = $rootScope.configuratorServer + '/api/model_cache/?responseFormat=json';			
            return $http.get(url, {
				headers: {'Cache-Control':'no-cache', 'Pragma':'no-cache'}
			});

			
        }
		
		function clearModelCache() {			
            var url = $rootScope.configuratorServer + '/api/model_cache'; 
			return $http.delete(url);
			
        }
		
		function clearModelCacheEntries() {			
            var url = $rootScope.configuratorServer + '/api/model_cache/entries'; 
			return $http.delete(url);			
        }


    }
})();
