/*
 * IBM Confidential 
 *
 * OCO Source Materials
 *
 * IBM Configure, Price , Quote (5725 D11)
 *
 * (c) Copyright IBM Corp. 2016 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets,
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */

(function() {
	'use strict';

	angular.module('ibmConfiguratorApp.shell').controller('regressionController', RegressionController);

	RegressionController.$inject = [ 'configuratorService', '$scope', 'logger'];
	/* @ngInject */

	function RegressionController(configuratorService, $scope, logger) {
		$scope.testId;
		$scope.groupId;
		$scope.description;
		$scope.message;
        $scope.requestFormat='XML';

		$scope.trapTestcase = function() {
			configuratorService.trapTestcase($scope.testId, $scope.groupId,
					$scope.description).success(function(response) {
				$scope.message = response.responseDetails.responseMessage;
				$('#trapModalClose').trigger('click');
				alert($scope.message);
			});
		};
		
		$scope.findTestcase = function() {
			configuratorService.findTestcase($scope.regressionFindTestId, $scope.regressionFindGroupId).success(function(response) {
				$scope.message = response.responseDetails.responseMessage;
                $scope.testCases = response.responseDetails.testCaseList.testCase;
                if ($scope.testCases) {
                    for(var i=0; i<$scope.testCases.length; i++) {
                        configuratorService.generalRegressionRequest('GET_JSON_REPORT', 'JSON', $scope.testCases[i].testId, $scope.testCases[i].groupId, $scope.testCases[i].testKey)
                            .success(function(response) {
                                $scope.testCases[i].problems = response.responseDetails.report.totalProblem;
                            });
                    }
                }
			});
		};

        $scope.executeTestcase = function() {
            if ($scope.testCases) {
                for(var i=0; i<$scope.testCases.length; i++) {
                    if($scope.testCases[i].checked) {
                        configuratorService.executeTestcase($scope.testCases[i].testId, $scope.testCases[i].groupId,$scope.testCases[i].testKey)
                            .success(function(response) {
                                $scope.message = response.responseDetails.responseMessage;
                                alert(response.testId + 'has been executed successfully');
                                $scope.findTestcase();
                            });
                    }
                }
            }
        };

        $scope.rebase = function() {
            if ($scope.testCases) {
                for(var i=0; i<$scope.testCases.length; i++) {
                    if($scope.testCases[i].checked) {
                        configuratorService.rebase($scope.testCases[i].testId, $scope.testCases[i].groupId,$scope.testCases[i].testKey)
                            .success(function(response) {
                                $scope.message = response.responseDetails.responseMessage;
                                alert(response.testId + 'has been rebased successfully');
                                $scope.findTestcase();
                            });
                    }
                }
            }
        };

        $scope.deleteTestcase = function() {
            if ($scope.testCases) {
                for(var i=0; i<$scope.testCases.length; i++) {
                    if($scope.testCases[i].checked) {
                        configuratorService.deleteTestcase($scope.testCases[i].testId, $scope.testCases[i].groupId,$scope.testCases[i].testKey)
                            .success(function(response) {
                                $scope.message = response.responseDetails.responseMessage;
                                alert(response.testId + 'has been deleted successfully');
                                $scope.findTestcase();
                            });
                    }
                }
            }
        };

        $scope.checkAll = function() {
            if ($scope.testCases) {
                for(var i=0; i<$scope.testCases.length; i++) {
                    $scope.testCases[i].checked = $scope.checkall;
                }
            }
        };

        $scope.generalRegressionRequest = function(method, testId, groupId, testKey) {
            configuratorService.generalRegressionRequest('GET_'+ $scope.requestFormat+'_'+method, $scope.requestFormat, testId, groupId, testKey)
                .success(function(response) {
                    $scope.generalTitle = method;
                    if($scope.requestFormat ==='JSON') {
                        $scope.generalResponse = JSON.stringify(response.responseDetails[method.toLowerCase()]);
                    } else {
                        var xmlDoc = $.parseXML(response);
                        var $xml = $(xmlDoc);
                        $scope.generalResponse = $xml.text().trim();
                    }
                    $('#regressionGeneralModal').modal('show');

            });
        };

        $scope.debug = function(testId, groupId, testKey) {
            configuratorService.generalRegressionRequest('DEBUG', $scope.requestFormat, testId, groupId, testKey)
                .success(function(response) {
                    $scope.generalTitle = 'DEBUG';
                    if($scope.requestFormat ==='JSON') {
                        $scope.generalResponse = JSON.stringify(response.responseDetails.debugTrace);
                    } else {
                        var headIndex = response.indexOf('<ResponseDetails>');
                        response = response.substring(headIndex, response.length);
                        response = response.replace('<ResponseDetails>','');
                        response = response.replace('</ManageModelRegression>','');
                        response = response.replace('</ResponseDetails>','');
                        $scope.generalResponse = response.trim();
                    }
                    $('#regressionGeneralModal').modal('show');
            });
        };
	}
})();
