/*
 * IBM Confidential 
 *
 * OCO Source Materials
 *
 * IBM Configure, Price , Quote (5725 D11)
 *
 * (c) Copyright IBM Corp. 2015, 2016 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets,
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */

angular.module('ibmConfiguratorApp.configurator.widget').controller('WidgetController', ['$scope', '$rootScope', '$filter', '$translate', 'configuratorService', function ($scope, $rootScope, $filter, $translate, configuratorService) {
    'use strict';
    $scope.none = {
        value: 'none'
    };

    $scope.showAs = {
        value: 'table'
    };

    $scope.showItemImageFocus = {
        value: 'false'
    };

    // UEV Date picker
    $scope.dateOptions = {
        dateDisabled: disabled,
        formatYear: 'yy',
        maxDate: new Date(2065, 11, 30),
        minDate: new Date(),
        startingDay: 1
    };

    // Disable weekend selection
    function disabled(data) {
        var date = data.date, mode = data.mode;
        return mode === 'day' && (date.getDay() === 0 || date.getDay() === 6);
    }

    $scope.getDropdownOptions = function (option) {
        var opts = [];
        var index = 0;

        var style = this.getValue(option.properties['UI: PRICING STYLE']);

        if (!this.hideNoneSelection(option)) {
            var desc = 'None';

            if (option.selected !== undefined) {
                var tmpopt;
                var deltaPrice = this.formatExclusivePrice(tmpopt, option, option.selected.id);
                desc += ' ' + deltaPrice;
            }

            opts[index++] = {
                'id': 'none',
                'value': desc,
                'qty': '0'
            };
        }
        for (var i=0; i<option.options.length; i++) {
            if (!this.isDisplayable(option.options[i], option)) {
                continue;
            }

            this.initQty(option.options[i]);

            var description = '';
            if (this.isConstraint(option.options[i])) {
                var constraintType = this.getValue(option.options[i].properties['_constraintType']);
                if (constraintType === 0) {
                    description += '* ';
                } else if (constraintType === 1) {
                    description += '! ';
                } else if (constraintType === 2) {
                    description += '~ ';
                }
            }

            description += this.getDescription(option.options[i], option);

            if (option.selected !== undefined) {
                var deltaPrice = this.formatExclusivePrice(option.options[i], option, option.selected.id);

                description += ' ' + deltaPrice;
            }

            opts[index++] = {
                'id': option.options[i].path,
                'value': description,
                'qty': option.options[i].quantity,
                'option': option.options[i],
                'unitPrice': this.getOptionPrice(option.options[i])
            };
        }
        return opts;
    };

    $scope.initDate = function (opt) {
        if (typeof (opt.uEVSelection) !== 'undefined' && opt.uEVSelection !== null && opt.uEVSelection !== '') {
            opt.uevDate = new Date(opt.uEVSelection);
        }
    };

    $scope.isEmpty = function (value) {
        var empty = false;
        if (typeof (value) === 'undefined' || value === null || value.length === 0) {
            empty = true;
        }
        return empty;
    };

    $scope.hasEmptyValue = function (value) {
        var empty = true;
        if (typeof (value) !== 'undefined') {
            empty = (value.length === 0);
        }
        return empty;
    };

    $scope.isTabHidden = function (tabName, model) {
        //UI: HIDE TABS
        var flag = false;
        var hideTabs = this.getValue(model.properties['UI: HIDE TABS']);
        if ('' !== hideTabs) {
            var tabs = hideTabs.split(';');
            for (var i in tabs) {
                if (tabName === tabs[i]) {
                    flag = true;
                    break;
                }
            }
        }
        return flag;
    };

    $scope.getOptionPrice = function (option) {
        var qtyValue = this.getValue(option.properties['_quantity']);
        if ('' === qtyValue) {
            qtyValue = '0';
        }
        var qty = parseInt(qtyValue);
        if (qty < 1) {
            qty = 1;
        }

        var up = 0;
        var totalItemPrice = this.getValue(option.properties['_totalItemPrice']);
        if (totalItemPrice !== '') {
            up = parseFloat(totalItemPrice, 10);
        } else {
            up = parseFloat(option.properties['UI: PRICE'].value, 10);
        }
        return up;
    };

    $scope.initDropDownlist = function (option) {
        var optPath = this.getCurrentSelection(option);
        var opts = this.getDropdownOptions(option);
        for (var i in opts) {
            if (optPath === opts[i].id) {
                option.selected = opts[i];
            }
        }
    };

    $scope.getDefaultDropdownOption = function (option) {
        var optPath = '';

        for (var i=0; i<option.options.length; i++) {
            var defSel = option.options[i].properties['UI: DEFAULT SELECTION'].value;
            if ('yes' === defSel) {
                optPath = option.options[i].path;
            }
        }

        var opts = this.getDropdownOptions(option);
        for (var i in opts) {
            if (optPath === opts[i].id) {
                return opts[i];
            }
        }
    };

    $scope.getOptionClass = function (options, path) {
        var filteredOptions = [];
        for (var i = 0; i < options.length; i++) {
            if (path === options[i].path) {
                filteredOptions.push(options[i]);
                break;
            }
        }

        // check for option class groups
        if (filteredOptions.length === 0) {
            for (var i = 0; i < options.length; i++) {
                if (options[i].path.indexOf(path + '.') === 0) {
                    filteredOptions.push(options[i]);
                }
            }
        }
        //console.log('getOptionClass', path, filteredOptions);
        return filteredOptions;
    };

    $scope.hasMessagesForOption = function (option, messages) {
        var hasMessages = false;
        if (messages) {
            for (var i=0; i<messages.length; i++) {
                if (this.stringContains(messages[i].path, option.path)) {
                    hasMessages = true;
                    break;
                }
            }
        }
        return hasMessages;
    };

    $scope.hasMessagesForModel = function (mdPath, messages) {
        var hasMessages = false;
        if (messages) {
            for (var i=0; i<messages.length; i++) {
                if (this.stringContains(messages[i].path, mdPath)) {
                    hasMessages = true;
                    break;
                }
            }
        }
        return hasMessages;
    };

    $scope.initQty = function (option) {
        if (option.quantity === undefined) {
            var qtyValue = this.getValue(option.properties['_quantity']);
            if ('' === qtyValue) {
                qtyValue = '0';
            }
            option.quantity = parseInt(qtyValue);
            if (option.quantity < 1) {
                option.quantity = 1;
            }
        }
    };

    $scope.stringContains = function (str1, str2) {
        return str1.indexOf(str2) > -1;
    };

    $scope.stringEquals = function (str1, str2) {
        return str1 === str2;
    };

    $scope.initCtrl = function (option) {
        $scope.initQty(option);

        var totalItemPrice = this.getValue(option.properties['_totalItemPrice']);
        if (totalItemPrice !== '') {
            option.unitPrice = parseFloat(totalItemPrice, 10);
            option.price = option.unitPrice * option.quantity;
        } else {
            option.unitPrice = parseFloat(option.properties['UI: PRICE'].value, 10);
            option.price = option.unitPrice * option.quantity;
        }

        return option.path;
    };

    $scope.isEmptyDate = function (dateString) {
        if (dateString === undefined || dateString === '' || dateString === '0') {
            return true;
        } else {
            return false;
        }
    };

    $scope.asArray = function (value) {
        if (value !== undefined) {
            return value.split(',');
        } else {
            return null;
        }
    };

    $scope.isSubmodel = function (opt) {
        var control = this.getValue(opt.properties['UI: CONTROL']);
        if (opt.properties['CONFIG: SUBMODEL NAME'] && control !== 'v2:dynamic-instantiation' && opt.elementType !== 'OPTION_CLASS') {
            return true;
        } else {
            return false;
        }
    };

    $scope.isEmptyValue = function (value) {
        return (!value || angular.isUndefined(value));
    };

    $scope.getAllowedUEVValues = function (option) {
        var uevType = this.getValue(option.properties['UI: UEV TYPE']);
        if (uevType === 'numeric') {
            var allowedValues = this.getValue(option.properties['UI: UEV ALLOWED VALUES']);
            if (allowedValues !== '') {
                var n = allowedValues.indexOf('-');
                var qtys = '';
                if (n > 0) {
                    var start = parseFloat(allowedValues.substring(0, n));
                    var end = parseFloat(allowedValues.substring(n + 1, allowedValues.length));
                    for (var i = start; i <= end; i++) {
                        if (qtys !== '') {
                            qtys += ',';
                        }
                        qtys += i + '.0';
                    }
                    return this.asArray(qtys);
                } else {
                    return this.asArray(allowedValues);
                }
            }
        } else if (uevType === 'integer') {
            var allowedValues = this.getValue(option.properties['UI: UEV ALLOWED VALUES']);
            if (allowedValues !== '') {
                var n = allowedValues.indexOf('-');
                var qtys = '';
                if (n > 0) {
                    var start = parseInt(allowedValues.substring(0, n));
                    var end = parseInt(allowedValues.substring(n + 1, allowedValues.length));
                    for (var i = start; i <= end; i++) {
                        if (qtys !== '') {
                            qtys += ',';
                        }
                        qtys += i;
                    }
                    return this.asArray(qtys);
                } else {
                    return this.asArray(allowedValues);
                }
            }
        } else {
            var allowedValues = this.getValue(option.properties['UI: UEV ALLOWED VALUES']);
            return this.asArray(allowedValues);
        }
    };

    $scope.getAllowedQty = function (option) {
        var alowQty = this.getValue(option.properties['UI: POPUP-QTY ALLOWED VALUES']);
        if (alowQty !== '') {
            var n = alowQty.indexOf('-');
            var qtys = '';
            if (n > 0) {
                var start = parseInt(alowQty.substring(0, n));
                var end = parseInt(alowQty.substring(n + 1, alowQty.length));
                for (var i = start; i <= end; i++) {
                    if (qtys !== '') {
                        qtys += ',';
                    }
                    qtys += i;
                }
                return this.asArray(qtys);
            } else {
                return this.asArray(alowQty);
            }
        }
    };

    $scope.getAllowedQtySelectOptions = function (option) {
        var selOptions = this.getAllowedQty(option);
        if (selOptions) {
            for (var i=0; i<selOptions.length; i++) {
                selOptions[i] = parseInt(selOptions[i], 10);
            }
        }
        return selOptions;
    };

    $scope.getValue = function (obj) {
        if (obj !== undefined && obj.value !== undefined) {
            return obj.value;
        } else {
            return '';
        }
    };

    $scope.initOptions = function (option) {
        option.selected = $scope.isSelectedInPicks(option);
    };

    $scope.useAllowedQuantity = function (option) {
        var value = this.getValue(option.properties['UI: POPUP-QTY ALLOWED VALUES']);
        if ('' === value) {
            return false;
        } else {
            return true;
        }
    };

    $scope.displayQuantity = function (option) {
        var value = this.getValue(option.properties['UI: OPTION CLASS VIEW']);
        if ('POPUP-QTY' === value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.isInvisible = function (option) {
        var value = this.getValue(option.properties['UI: OPTION CLASS VIEW']);
        if ('INVISIBLE' === value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.showTabGuidingText = function (tab) {
        if ('' !== tab.guidingText) {
            return true;
        } else {
            return false;
        }
    };

    $scope.hideNoneSelection = function (option) {
        var value = this.getValue(option.properties['UI: SUPPRESS NONE SELECTION']);
        if ('yes' === value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.isSelectedInPicks = function (option) {
        var pickSelected = false;
        for (var i=0; i<$rootScope.picks.length; i++) {
            if (option.path === $rootScope.picks[i].item) {
                pickSelected = true;
                break;
            }
        }
        return pickSelected;
    };

    $scope.getCurrentQuantity = function (option) {
        var opts = option.options;
        var qty = 1;
        var qtySet = false;
        if (opts) {
            for (var i=0; i<opts.length; i++) {
                for (var j=0; j<$rootScope.picks.length; j++) {
                    if (opts[i].path === $rootScope.picks[j].item) {
                        qty = $rootScope.picks[j].quantity;
                        qtySet = true;
                        break;
                    }
                }
                if (qtySet) {
                    break;
                }
            }
        }
        return qty;
    };

    $scope.getCurrentSelection = function (option) {
        var opts = option.options;
        var defSel = '';

        if ($scope.ctabindex === undefined) {
            $scope.ctabindex = 0;
        }

        if (opts) {
            for (var i=0; i<opts.length; i++) {
                // if _quantity > 0, its selected.
                // This is to handle rule pick scenario (rule picks are not present in picks)
                if (parseInt(opts[i].properties._quantity.value) > 0) {
                    defSel = opts[i].path;
                } else {
                    for (var j=0; j<$rootScope.picks.length; j++) {
                        if (opts[i].path === $rootScope.picks[j].item) {
                            defSel = opts[i].path;
                            break;
                        }
                    }
                }
                if ('' !== defSel) {
                    break;
                }
            }
        }
        if ('' === defSel) {
            defSel = 'none';
        }

        return defSel;
    };

    $scope.isRecommended = function (option) {
        var value = this.getValue(option.properties['UI: RECOMMEND']);
        if ('yes' === value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.isRulePicked = function (option) {
        var value = this.getValue(option.properties['_rulePick']);
        if ('1' === value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.isConstraint = function (option) {
        var value = this.getValue(option.properties['_isConstrained']);
        if ('1' === value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.isResolvableConstraint = function (option) {
        var constraintType = this.getValue(option.properties['_constraintType']);
        var resolvableConstraint = this.getValue(option.properties['_resolvableConstraint']);
        if ('2' === constraintType) {
            if ('yes' === resolvableConstraint || 'true' === resolvableConstraint) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    };

    $scope.getDescription = function (option, parent) {
        var desc = '';
        desc = this.getValue(option.properties['_expandedDescription']);

        if ('' === desc) {
            // old code - without expand/gather
            if (parent !== undefined && parent !== null) {
                desc = this.getValue(parent.properties['UI: ITEM DISPLAY NAME']);
            }

            if ('' === desc) {
                desc = this.getValue(option.properties['UI: DISPLAY NAME']);
                if ('' === desc) {
                    desc = this.getValue(option.properties['_description']);
                }
            }
        }
        return desc;
    };

    $scope.isItemDisabled = function (option) {
		if(option==undefined){
			return false;
		}
        var obj = option.properties['UI: ITEM DISABLED'];
        if (obj !== undefined && obj.value !== undefined && 'yes' === obj.value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.isInTab = function (option, classList) {
        var classArr = [];
        for (var idx = 0; idx < classList.length; idx++) {
            classArr.push(classList[idx].class);
        }

        for (var j = 0; j < classArr.length; j++) {
            if (option.path.indexOf(classArr[j]) === 0) {
                return true;
            }
        }

        return false;
    };

    $scope.isDisplayable = function (option, optionClass) {
        var val1 = option.properties['_amEntitled'].value;
        var val2 = option.properties['_isViewable'].value;

        if ('1' !== val1 || '1' !== val2) {
            return false;
        }

        if (optionClass !== undefined && optionClass != null) {
            if (('yes' === this.getValue(optionClass.properties['UI: PREVENT SELECTION'])) && this.isConstraint(option) && ('2' === this.getValue(option.properties['_constraintType'])) && ('0' === this.getValue(option.properties['_quantity']))) {
                return false;
            }
        }

        return true;
    };

    $scope.isOCDisplayable = function (optionClass) {
        var val3 = optionClass.properties['_isViewable'].value;
        if ('1' !== val3) {
            return false;
        }
        return true;
    };

    $scope.getTabularDisplayProperties = function (option) {
        var attrName = null;
        var propName = null;

        if (option !== undefined && option.properties['UI: COLUMN HEADINGS'] !== undefined && option.properties['UI: COLUMN PROPERTIES'] !== undefined) {
            attrName = option.properties['UI: COLUMN HEADINGS'].value.split(';');
            propName = option.properties['UI: COLUMN PROPERTIES'].value.split(';');

            option.tabularProperties = attrName.map(function (value, index) {
                return {
                    tabName: value,
                    tabPropName: propName[index]
                };
            });
        } else {
            return null;
        }
    };


    $scope.resolveItem = function (optionPath) {
        if (optionPath === undefined || optionPath === null) {
            return;
        }

        configuratorService.resolveConstrainedItem(optionPath, this.parentConfigurations, this.nestedConfiguration, $rootScope.modelDepth).success(function (response) {

            var items = $scope.findItemAndParent(optionPath, response.options);
            var resolveParentItem = items.parent;
            var resolveItem = items.child;

            $rootScope.resolveItemDesc = resolveItem.properties['_description'].value;

            var recDomain = resolveItem.properties['_recommendationDomain'];
            var recDomainValues = recDomain.value.split(/\r?\n/);

            var recommendationRows = [];
            var count = 0;

            for (var i = 0; i < resolveItem.properties['_recommendationCount'].value; i++) {
                var rec = resolveItem.properties['_recommendation' + '[' + i + ']'];
                var recValues = rec.value.split(/\r?\n/);

                var isDisabled = false;
                for (var j=0; j<recValues.length; j++) {
                    var tmpOption = $scope.findItem(recValues[j], response.options);
                    if (tmpOption !== undefined) {
                        isDisabled = $scope.isItemDisabled(tmpOption);
                        if (isDisabled) {
                            break;
                        }
                    }
                }

                if (isDisabled) {
                    continue;
                }

                recommendationRows[count] = [];
                recommendationRows[count]['keepList'] = $scope.buildKeepList(recValues, resolveItem, resolveParentItem, response.options);

                recommendationRows[count]['removeList'] = $scope.buildRemoveList(recValues, recDomainValues, response.options);

                recommendationRows[count]['addList'] = $scope.buildAddList(recValues, recDomainValues, response.options, recommendationRows[i]['keepList']);

                recommendationRows[count].name = 'recRow' + count;

                count++;
            }

            $rootScope.showingArea = 'Constraints';

            $rootScope.recommendationRows = recommendationRows;

        });
    };

    $scope.buildKeepList = function (recList, resolveItem, resolveParentItem, options) {
        var keepList = [];
        keepList[0] = {};

        keepList[0]['Parent'] = resolveParentItem;

        var childList = [];
        childList[0] = resolveItem;

        keepList[0]['Children'] = childList;

        for (var i=0; i<recList.length; i++) {
            var recPath = recList[i];
            var recItems = this.findItemAndParent(recPath, options);
            var recChildItem = recItems.child;

            if (resolveItem.id !== recChildItem.id && recChildItem.properties['_quantity'].value > 0) {
                var recParentItem = recItems.parent;
                var foundInList = false;

                var index = this.searchParentPathInConstraintList(recParentItem.path, keepList);
                if (index > -1) {
                    var currentArray = keepList[index]['Children'];
                    currentArray[currentArray.length] = recChildItem;
                } else {
                    var length = keepList.length;
                    keepList[length] = {};
                    keepList[length]['Parent'] = recParentItem;
                    var tempList = [];
                    tempList[0] = recChildItem;
                    keepList[length]['Children'] = tempList;
                }
            }

        }

        return keepList;
    };

    $scope.buildRemoveList = function (recList, recDomainList, options) {
        var removeList = [];

        for (var i=0; i<recDomainList.length; i++) {
            var recDomainPath = recDomainList[i];

            if (this.isInList(recDomainPath, recList) === false) {
                var recDomainItems = this.findItemAndParent(recDomainPath, options);
                var recDomainParentItem = recDomainItems.parent;
                var recDomainChildItem = recDomainItems.child;

                var index = this.searchParentPathInConstraintList(recDomainParentItem.path, removeList);
                if (index > -1) {
                    var currentArray = removeList[index]['Children'];
                    currentArray[currentArray.length] = recDomainChildItem;
                } else {
                    var length = removeList.length;
                    removeList[length] = {};
                    removeList[length]['Parent'] = recDomainParentItem;
                    var tempList = [];
                    tempList[0] = recDomainChildItem;
                    removeList[length]['Children'] = tempList;
                }
            }
        }

        return removeList;
    };

    $scope.buildAddList = function (recList, recDomainList, options, keepList) {
        var addList = [];

        for (var i=0; i<recList.length; i++) {
            var recPath = recList[i];
            if (!this.isChildPathInConstraintList(recPath, keepList) && this.isInList(recPath, recDomainList) === false) {
                var recItems = this.findItemAndParent(recPath, options);
                var recParentItem = recItems.parent;
                var recChildItem = recItems.child;

                var index = this.searchParentPathInConstraintList(recParentItem.path, addList);
                if (index > -1) {
                    var currentArray = addList[index]['Children'];
                    currentArray[currentArray.length] = recChildItem;
                } else {
                    var length = addList.length;
                    addList[length] = {};
                    addList[length]['Parent'] = recParentItem;
                    var tempList = [];
                    tempList[0] = recChildItem;
                    addList[length]['Children'] = tempList;
                }
            }
        }

        return addList;
    };

    $scope.isInList = function (name, nameList) {
        for (var i=0; i<nameList.length; i++) {
            if (name === nameList[i]) {
                return true;
            }
        }

        return false;
    };

    $scope.searchParentPathInConstraintList = function (parentPath, constraintList) {
        for (var i=0; i<constraintList.length; i++) {
            if (constraintList[i]['Parent'].path === parentPath) {
                return i;
            }
        }
        return -1;
    };

    $scope.isChildPathInConstraintList = function (childPath, constraintList) {
        for (var i=0; i<constraintList.length; i++) {
            var childrenList = constraintList[i]['Children'];
            if (childrenList !== undefined) {
                for (var j=0; j<childrenList.length; j++) {
                    if (childrenList[j].path === childPath) {
                        return true;
                    }
                }
            }
        }
        return false;
    };

    $scope.findItem = function (path, options) {
        if (options === undefined || options === null) {
            return;
        }

        for (var i=0; i<options.length; i++) {
            if (path === options[i].path) {
                return options[i];
            } else {
                var result = this.findItem(path, options[i].options);
                if (result === undefined || result === null) {
                    continue;
                } else {
                    return result;
                }
            }
        }
    };

    $scope.findItemAndParent = function (path, options, parentItem) {
        if (options === undefined || options === null) {
            return;
        }

        for (var i=0; i<options.length; i++) {
            if (path === options[i].path) {
                var obj = {};
                obj.parent = parentItem;
                obj.child = options[i];
                return obj;
            } else {
                var result = this.findItemAndParent(path, options[i].options, options[i]);
                if (result === undefined || result === null) {
                    continue;
                } else {
                    return result;
                }
            }
        }
    };

    $scope.constantTextDisplayale = function (option) {
        if (option === undefined || option === null) {
            return false;
        }
        var value = this.getValue(option.properties['UI: CONSTANT GUIDING TEXT']);
        if ('' === value) {
            return false;
        } else {
            return true;
        }
    };

    $scope.preCheckTextDisplayable = function (option) {
        if (option === undefined || option === null) {
            return false;
        }
        var value = this.getValue(option.properties['UI: PRE PICK GUIDING TEXT']);
        if ('' === value) {
            return false;
        } else {
            return true;
        }
    };

    $scope.postCheckTextDisplayable = function (option) {
        if (option === undefined || option === null) {
            return false;
        }
        var value = this.getValue(option.properties['UI: POST PICK GUIDING TEXT']);
        if ('' === value) {
            return false;
        } else {
            return true;
        }
    };

    $scope.helpURLDisplayable = function (option) {
        if (option === undefined || option === null) {
            return false;
        }
        var value = this.getValue(option.properties['UI: HELP URL']);
        if ('' === value) {
            return false;
        } else {
            return true;
        }
    };

    $scope.helpTextDisplayable = function (option) {
        if (option === undefined || option === null) {
            return false;
        }
        var value = this.getValue(option.properties['UI: HELP TEXT0']);
        if ('' === value) {
            return false;
        } else {
            return true;
        }
    };

    $scope.showItemImage = function (option) {
        if (option === undefined || option === null) {
            return false;
        }
        var value = this.getValue(option.properties['UI: SHOW ITEM IMAGES']);
        if ('yes' === value) {
            return true;
        } else {
            return false;
        }
    };

    $scope.showOptonClassDesc = function (option) {
        if (option === undefined || option === null) {
            return false;
        }
        var value = this.getValue(option.properties['UI: SUPPRESS NAME DISPLAY']);
        if ('yes' === value) {
            return false;
        } else {
            return true;
        }
    };

    $scope.formatExclusivePrice = function (opt, option, selectedOpt) {
        var price = '';

        if (opt !== undefined) {
            if (opt.price !== undefined) {
                price = opt.price;
            } else {
                price = this.getOptionPrice(opt) * opt.quantity;
            }
        }

        var style = this.getValue(option.properties['UI: PRICING STYLE']);

        if (style === undefined || style === 'ABSOLUTE' || style === '') {
            price = $filter('currency')(price, $rootScope.currencySymbol);
            return price;
        } else if (style !== undefined && style === 'NONE') {
            // Price style is NONE
            return '';
        } else if (style !== undefined && style === 'DELTA') {
            var refPrice = 0;
            var item;
            if (selectedOpt !== undefined && option.options !== undefined) {
                item = this.findItem(selectedOpt, option.options);
                if (item !== undefined) {
                    if (item.price !== undefined) {
                        refPrice = item.price;
                    } else {
                        refPrice = this.getOptionPrice(item) * item.quantity;
                    }
                }
            }

            if ((opt === undefined || opt === null) && (item === undefined || item === null)) {
                return '';
            }

            if (opt !== undefined && opt !== null && item !== undefined && item !== null && (opt.id === item.id)) {
                return '';
            }

            var delta = price - refPrice;

            price = $filter('currency')(Math.abs(delta), $rootScope.currencySymbol);

            if (delta < 0) {
                price = $translate.instant('TXT_SUBTRACT') + ' ' + price;
            } else {
                price = $translate.instant('TXT_ADD') + ' ' + price;
            }

            return price;
        }
    };
	
	/*
	 * This function returns the number of child options
	 * belonging to a given option class. This primarily used
	 * in dynamic instantiation control to check whether
	 * 'Add' button should be disabled based on the property
	 * 'CONFIG: MAX INSTANCES'.
	 */
	$scope.getChildOptionsCount = function (option) {
		var childOptions = option.options;
		if(childOptions) {
			return childOptions.length;
		}
		return 0;
	}
}]);
