/*
 * Licensed Materials - Property of IBM
 * IBM Sterling Configure Price Quote (5725-D11)
 * (C) Copyright IBM Corp.2015-2017 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function () {
	'use strict';

	angular.module('ibmConfiguratorApp.configurator.widget').directive('ibmConfiguratorWidget', ibmConfiguratorWidget);

	ibmConfiguratorWidget.$inject = ['configuratorService', '$compile', '$timeout', '$rootScope'];
	/* @ngInject */
	function ibmConfiguratorWidget(configuratorService, $compile, $timeout, $rootScope) {
		// Usage:
		var getTemplate = function (option, widgets) {
			var resolvedWidget = '';
			var uiControl = option.properties['UI: CONTROL'];
			if (uiControl === undefined || uiControl.value === undefined) {

				var modVersion = option.properties['_modelVersion'];
				if (modVersion === undefined || modVersion.value === undefined) {
					// old models
					uiControl = {
						'value': 'controls/radio.jsp'
					};
				} else {
					uiControl = {
						'value': 'v2:single-select'
					};
				}
			}
			if (uiControl) {
				var recommendedControl = option.properties['UI: CONTROL RECOMMENDATION'];
				if (recommendedControl) {
					resolvedWidget = widgets[uiControl.value].artifacts[recommendedControl.value];
				} else {
					resolvedWidget = widgets[uiControl.value].artifacts[widgets[uiControl.value].default];
				}
			}

			return '<ng-include src=\"\'' + resolvedWidget + '\'\"></ng-include>';
			/* return '<ng-include src='getWidget()'></ng-include>'; */
		};

		var directive = {
			restrict: 'AE',
			scope: {
				//modelhierarchy:'=modelhierarchy',
				//modelproperties:'=modelproperties',
				parentConfigurations: '=parentconfigurations',
				widgets: '=widgets',
				option: '=option',
				summary: '=summary',
				picks: '=picks',
				messages: '=messages',
				//tabs: '=tabs',
				ctabindex: '=ctabindex',
				options: '=options',
				titleDisplay: '=titledisplay',
				setData: '&'
			},
			link: link
		};
		return directive;

		function link(scope, element, attrs) {
			scope.wrapperSetData = function (response) {
				scope.setData({
					response: response
				});
			};

			scope.updatePicks = updatePicks;
			scope.updatePicksForMultiList = updatePicksForMultiList;
			scope.addPick = addPick;
			scope.manualUpdate = manualUpdate;
			scope.removePick = removePick;
			scope.removePicks = removePicks;
			scope.changeQuantity = changeQuantity;
			scope.updateDropDown = updateDropDown;
			scope.updateSubModelDetails = updateSubModelDetails;
			scope.instantiateSubModel = instantiateSubModel;
			scope.copySubModel = copySubModel;
			scope.removeSubModel = removeSubModel;

			scope.radioValue = '';
			//            scope.selectedOptWithTab = '';
			//            $rootScope.currentFocusId = '';
			scope.showErrors = false;
			scope.showWarnings = false;
			scope.showSuggestions = false;
			scope.currentMessage = [];
			if (scope.messages) {
				for (var i = 0; i < scope.messages.length; i++) {
					if (scope.messages[i].path === scope.option.path) {
						scope.currentMessage.push({
							msg: scope.messages[i].message,
							msgType: scope.messages[i].messageType
						});
						if (scope.messages[i].messageType === 'Error') {
							scope.showErrors = true;
						} else if (scope.messages[i].messageType === 'Warning') {
							scope.showWarnings = true;
						} else {
							scope.showSuggestions = true;

						}

					}
				}
			}

			var el = $compile(getTemplate(scope.option, scope.widgets))(scope);
			element.replaceWith(el);

			function updatePicks(opt, event, option) {
				if (event !== undefined) {
					if (event.target.checked) {
						addPick(opt, option, true);
						//opt.selected = true;
					} else {
						removePick(opt, option, true);
						//opt.selected = false;
					}
				}
			}

			function updatePicksForMultiList(opt, option) {
				if ((opt.selected === undefined && isSelectedInPicks(opt)) || (opt.selected !== undefined && opt.selected)) {
					removePick(opt, option, true);
					opt.selected = false;
				} else if (opt.selected === undefined || !opt.selected) {
					addPick(opt, option, true);
					opt.selected = true;
				} else {
					removePick(opt, option, true);
					opt.selected = false;
				}
			}

			function isSelectedInPicks(option) {
				var pickSelected = false;
				for (var i = 0; i < $rootScope.picks.length; i++) {
					var pickPath = $rootScope.picks[i].item;
					var pickQty = $rootScope.picks[i].quantity;
					var pickVal = '';
					if ($rootScope.picks[i].value !== undefined) {
						pickVal = $rootScope.picks[i].value;
					}

					var optVal = option.uEVSelection;
					if (optVal === undefined) {
						optVal = '';
					}
					if (option.path === pickPath && option.quantity === pickQty && optVal === pickVal) {
						pickSelected = true;
						break;
					}
				}
				return pickSelected;
			}

			function changeQuantity(opt, option, isMultiSelection) {

				var isMulti = false;
				if (isMultiSelection !== undefined && isMultiSelection === 'true') {
					isMulti = true;
				}

				addPick(opt, option, isMulti);
			}

			function removeMultiOptionForManualUpdate(opt, option) {
				for (var i = 0; i < $rootScope.optArray.length; i++) {
					if ($rootScope.optArray[i].id === option.id) {
						// found in existing optArray means this option had been added before
						// remove opt from add array (if exists)
						if ($rootScope.optArray[i].add) {
							for (var j = 0; j < $rootScope.optArray[i].add.length; j++) {
								if ($rootScope.optArray[i].add[j].path === opt.path) {
									// found opt in existing add array, need to remove it from the add array
									$rootScope.optArray[i].add.splice(i, 1);
									break;
								}
							}
						}

						// add opt in remove array (if does not exist yet)
						// only need to add it into remove array when it's selected in server side
						if (isSelectedInPicks(opt)) {
							var existed = false;
							if ($rootScope.optArray[i].remove) {
								for (var j = 0; j < $rootScope.optArray[i].remove.length; j++) {
									if ($rootScope.optArray[i].remove[j].path === opt.path) {
										existed = true;
										break;
									}
								}
							}

							if (!existed) {
								$rootScope.optArray[i].remove.push(opt);
							}

						}
						return;
					}
				}

				// not found means this is a new one
				if (isSelectedInPicks(opt)) {
					var optObject = {};
					optObject.id = option.id;
					optObject.add = [];
					optObject.remove = [];
					optObject.remove.push(opt);
					$rootScope.optArray.push(optObject);
				}
			}

			function removePick(opt, option, isMultiSelection) {
				if ($rootScope.autoUpdate !== 'NONE') {

					/**
					 * fix for following defect
					 * 529696: Screen does not refresh on unchecking OI
					 **/
					if (isSelectedInPicks(opt) || (opt.properties._rulePick.value === "1")) {
						configuratorService.removeConfigurationPicks(opt, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth).success(handleConfigurationResponse);
					}
				}
				// currently only deals with multi selection, may need to support single selection later
				else if (isMultiSelection !== undefined && isMultiSelection) {
					removeMultiOptionForManualUpdate(opt, option);
				}
			}

			function handleConfigurationResponse(response) {
				$rootScope.picks = response.picks;
				$rootScope.inputProperties = response.inputProperties;
				scope.setData({
					response: response
				});
				$rootScope.modelPath = response.path;
				$rootScope.modelDepth = response.modelDepth;
				
				$rootScope.total = response.properties._totalPrice.value;
				$rootScope.traceLog = response.traceLog;
				$rootScope.unproceeedPropertyPool = response.propertyPool;
				$rootScope.propertyPool = preProcessPropertyPool(response.propertyPool);
				$rootScope.summary = response.summary;
				setFocus(scope.options);
				setAlert();
				if (response.properties['UI: DISPLAY NAME'] !== undefined) {
					if ('' !== response.properties['UI: DISPLAY NAME'].value) {
						$rootScope.modeldisname = response.properties['UI: DISPLAY NAME'].value;
					}
				} else {
					$rootScope.modeldisname = response.properties['_description'].value;
				}
			}

			function addSingleOptionForManualUpdate(opt, option) {
				for (var i = 0; i < $rootScope.optArray.length; i++) {
					if ($rootScope.optArray[i].id === option.id) {
						$rootScope.optArray[i].add = [];
						$rootScope.optArray[i].remove = [];

						// no need to add opt if it's already selected in server side
						if (!isSelectedInPicks(opt)) {
							$rootScope.optArray[i].add.push(opt);
						}

						return;
					}
				}

				// not found means this is a new one
				if (!isSelectedInPicks(opt)) {
					var optObject = {};
					optObject.id = option.id;
					optObject.add = [];
					optObject.remove = [];

					optObject.add.push(opt);
					$rootScope.optArray.push(optObject);
				}
			}

			function addMultiOptionForManualUpdate(opt, option) {
				for (var i = 0; i < $rootScope.optArray.length; i++) {
					if ($rootScope.optArray[i].id === option.id) {
						// remove opt from remove array (if exists)
						for (var j = 0; j < $rootScope.optArray[i].remove.length; j++) {
							if ($rootScope.optArray[i].remove[j].path === opt.path) {
								$rootScope.optArray[i].remove.splice(j, 1);

								break;
							}
						}

						// then add opt into add array (if does not exist yet)
						// no need to add opt if it's already selected in server side
						if (!isSelectedInPicks(opt)) {
							var existed = false;
							for (var j = 0; j < $rootScope.optArray[i].add.length; j++) {
								if ($rootScope.optArray[i].add[j].path === opt.path) {
									existed = true;

									break;
								}
							}

							if (!existed) {
								$rootScope.optArray[i].add.push(opt);
							}
						}

						return;
					}
				}

				// not found means this is a new one
				if (!isSelectedInPicks(opt)) {
					var optObject = {};
					optObject.id = option.id;
					optObject.add = [];
					optObject.remove = [];

					optObject.add.push(opt);
					$rootScope.optArray.push(optObject);
				}
			}

			function addPick(opt, option, isMultiSelection) {
				if ($rootScope.autoUpdate !== 'NONE') {
					if (!isSelectedInPicks(opt)) {
						configuratorService.addConfigurationPicks(opt, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth).success(handleConfigurationResponse);
					}
				} else if (isMultiSelection === undefined || !isMultiSelection) {
					// manually add opt for single selection
					addSingleOptionForManualUpdate(opt, option);
				} else {
					// manually add opt for multi selection
					addMultiOptionForManualUpdate(opt, option);
				}
			}

			function manualUpdate() {
				var toAdd = [];
				var toRemove = [];

				for (var i = 0; i < $rootScope.optArray.length; i++) {
					toAdd = toAdd.concat($rootScope.optArray[i].add);
					toRemove = toRemove.concat($rootScope.optArray[i].remove);
				}

				$rootScope.optArray = [];

				if (toRemove.length > 0 && toAdd.length > 0) {
					configuratorService.removeAndAddConfigurationPicks(toRemove, toAdd, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth).success(handleConfigurationResponse);
				} else if (toAdd.length > 0) {
					configuratorService.addConfigurationPicks(toAdd, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth).success(handleConfigurationResponse);
				} else if (toRemove.length > 0) {
					configuratorService.removeConfigurationPicks(toRemove, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth).success(handleConfigurationResponse);
				}
			}

			function hasPicksToRemove(option) {
				var opts = option.options;
				if (opts) {
					for (var i = 0; i < opts.length; i++) {
						if (isSelectedInPicks(opts[i])) {
							return true;
						}
					}
				}

				return false;
			}

			function removeOptionsForManualUpdate(option) {
				for (var i = 0; i < $rootScope.optArray.length; i++) {
					if ($rootScope.optArray[i].id === option.id) {
						$rootScope.optArray[i].add = [];
						$rootScope.optArray[i].remove = [];

						if (hasPicksToRemove(option)) {
							$rootScope.optArray[i].remove = $rootScope.optArray[i].remove.concat(option.options);
						}

						return;
					}
				}

				// not found means this is a new one
				if (hasPicksToRemove(option)) {
					var optObject = {};
					optObject.id = option.id;
					optObject.add = [];
					optObject.remove = [];
					optObject.remove = optObject.remove.concat(option.options);
					$rootScope.optArray.push(optObject);
				}
			}

			function removePicks(option) {
				if ($rootScope.autoUpdate !== 'NONE') {
					if (hasPicksToRemove(option)) {
						configuratorService.removeConfigurationPicks(option.options, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth)
							.success(handleConfigurationResponse);
					}
				} else {
					removeOptionsForManualUpdate(option);
				}

				option.selected = 'none';
			}

			function updateDropDown(option, parent, manualUpdate) {
				var addOpt = {
					'path': option.id,
					'quantity': option.qty,
					'value': ''
				};

				option.path = option.id;
				option.quantity = option.qty;

				if ('none' === option.id) {
					if ($rootScope.autoUpdate !== 'NONE') {
						configuratorService.removeConfigurationPicks(parent.options, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth)
							.success(handleConfigurationResponse);
					} else {
						removeOptionsForManualUpdate(parent);
					}
				} else {
					if ($rootScope.autoUpdate !== 'NONE') {
						if (!isSelectedInPicks(option)) {
							configuratorService.addConfigurationPicks(addOpt, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration, $rootScope.modelDepth)
								.success(handleConfigurationResponse);
						}
					} else {
						addSingleOptionForManualUpdate(option, parent);
					}
				}
			}

			function instantiateSubModel(option) {
				configuratorService.instantiateSubModel(option, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration)
					.success(handleConfigurationResponse)
					.catch(function (fail) {
						console.error(fail);
					});
			}

			function removeSubModel(option) {
				configuratorService.removeSubModel(option, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration)
					.success(handleConfigurationResponse)
					.catch(function (fail) {
						console.error(fail);
					});
			}

			function copySubModel(option) {
				configuratorService.copySubModel(option, $rootScope.picks, scope.parentConfigurations, scope.nestedConfiguration)
					.success(handleConfigurationResponse)
					.catch(function (fail) {
						console.error(fail);
					});
			}

			function updateSubModelDetails(punchedItem, modelDepth) {
				configuratorService.punchIntoSubModel($rootScope.picks, punchedItem, $rootScope.modelDepth, scope.parentConfigurations)
					.success(function (response) {
						$rootScope.picks = response.picks;
						$rootScope.inputProperties = response.inputProperties;
						scope.setData({
							response: response
						});
						$rootScope.modelPath = response.path;
						$rootScope.modelDepth = response.modelDepth;
						
						$rootScope.total = response.properties._totalPrice.value;
						$rootScope.passcurrentTab = $rootScope.currentTab;
						$rootScope.currentTab = 0;
						$rootScope.traceLog = response.traceLog;
						$rootScope.unproceeedPropertyPool = response.propertyPool;
						$rootScope.propertyPool = preProcessPropertyPool(response.propertyPool);
						$rootScope.summary = response.summary;
						setSubModelFocus();
						if (response.properties['UI: DISPLAY NAME'] !== undefined) {
							if ('' !== response.properties['UI: DISPLAY NAME'].value) {
								$rootScope.modeldisname = response.properties['UI: DISPLAY NAME'].value;
							}
						} else {
							$rootScope.modeldisname = response.properties['_description'].value;
						}

						if (response.properties['UI: AUTOMATIC POST'] !== undefined) {
							$rootScope.autoUpdate = response.properties['UI: AUTOMATIC POST'].value;
						} else {
							$rootScope.autoUpdate = 'UPDATE';
						}
					}).catch(function (fail) {
						console.error(fail);
					});
			}

			function setSubModelFocus() {
				$('#model_header').attr('tabindex', '-1').css('outline', 'none');
				$timeout(function () { // You might need this timeout to be sure its run after DOM render.
					$('#model_header').focus();
				}, 0, false);
			}

			function setAlert() {
				var hasMessages = false;
				if (scope.messages) {
					for (var i = 0; i < scope.messages.length; i++) {
						if (scope.messages[i].path.indexOf(scope.option.path) > -1) {
							hasMessages = true;
							break;
						}
					}
				}
				//console.log('has error:'+hasMessages);
				if (hasMessages) {
					$timeout(function () { // You might need this timeout to be sure its run after DOM render.
						// $('#alert_message').css('display', 'inline');
						$('#' + $rootScope.alertId).attr('role', '');
						$('#' + $rootScope.alertId).attr('role', 'alert');
					}, 0, false);
				}
			}

			function setFocus(options) {
				if (options) {
					for (var i = 0; i < options.length; i++) {
						if (options[i].options) {
							for (var j = 0; j < options[i].options.length; j++) {
								var isConstrained = options[i].options[j].properties['_isConstrained'];
								if (isConstrained !== undefined && '1' === isConstrained.value) {
									$timeout(function () {
										$('#constraint_' + options[i].options[j].path.replace(/[\.\"\s]/g, '_')).focus();
									}, 0, false);
									return;
								}
							}
						}
					}
				}

				var currentFocusId = $rootScope.currentFocusId.replace(/[\.\"\s]/g, '_');
				$timeout(function () { // You might need this timeout to be sure its run after DOM render.
					$('#' + currentFocusId).focus();
				}, 0, false);
			}

			function preProcessPropertyPool(propertyPool) {
				var newPropertyPool = [];
				if (propertyPool) {
					for(var i in propertyPool){
						if(propertyPool.hasOwnProperty(i)){
							for(var j in propertyPool[i]){
								if(propertyPool[i].hasOwnProperty(j)) {
									newPropertyPool.push( propertyPool[i][j]);
								}
							}
						}
					 }
			    }
				return newPropertyPool;
			}
		}
	}
})();