<?xml version="1.0" ?>
<!--
  Licensed Materials - Property of IBM 

  IBM Sterling Selling and Fullfillment Suite 9.1 

  (c) Copyright IBM Corp 2001, 2010 All Rights Reserved

  US Government Users Restricted Rights - Use, duplication or disclosure
  restricted by GSA ADP Schedule Contract with IBM Corp.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xslt"
    xmlns:cmgt="http://www.comergent.com/xslt"   
    version="1.0">

  

    <xsl:output method="text"/>
    <xsl:key name="signature" match="Method" use="@name" />
    <xsl:include href="../util/Constraints.xsl"/>
    <xsl:include href="../util/utils.xsl"/>
    
    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>
<!--
    <xsl:template match="*" mode="*"/>
-->
    <xsl:template match="JavaInterface">

<xsl:variable name="BizletName" select="@Name"/>
<xsl:if test="@Package">
package <xsl:value-of select="@Package"/>;
</xsl:if>

import java.io.*;
import java.util.*;

import com.comergent.dcm.bizlet.*;
import com.comergent.api.exception.*;


public class <xsl:value-of select="$BizletName"/>Invoker extends BizletInvoker implements <xsl:value-of select="$BizletName"/>
{
    private <xsl:value-of select="$BizletName"/> bizlet;
    private Object m_lastReturnValue=null;
    //private ObjectManager m_objectManager;

    public void init(Bizlet bizlet)
    {
        this.bizlet = (<xsl:value-of select="$BizletName"/>)bizlet;
        //m_objectManager=ObjectManager.getInstance();
    }

    /**
     * Invoke a Bizlet method, given name and parameter accessor
     */
    public BizletResponse invoke(String operationName, ParameterAccessor accessor) throws ComergentException
    {

        // TBD - Implement this method
        <xsl:for-each select="//JavaInterface/Method">
            <xsl:variable name="MethodName" select="@Name"/>

        if (operationName.equals(&quot;<xsl:value-of select="@Name"/>&quot;))
        {
            return invoke_<xsl:value-of select="$MethodName"/>(accessor);            
        }
        </xsl:for-each>
      
        // No match - can't invoke bizlet method
        throw new CannotInvokeBizletOperationException("<xsl:value-of select="//JavaInterface/@Name"/>:" + operationName);      
    }    

    // Start of Individual Methods for parameter Validation
    <xsl:apply-templates select="Method" mode="methodParamValidator"/>
    // End of Parameter validation methods
    
    
    //Start of Individual Method invoker section    
    <xsl:apply-templates select="Method" mode="methodInvoker"/>
    // End of Individual Method invoker section
    // Start of Paremeter Accessor Section
    //protected Hashtable parameters=null;

    public Object getObject(String className, String name) throws BizletException
    {
        try
        {
            return getObject(Class.forName(className),name);
        }
        catch(Exception ex)
        {
            throw new BizletException("failed get object", ex);
        }
    }
    
    public Object getObject(Class type, String name) throws BizletException
    {       
        Object value=null;
        //prepareParameterList();
        if(m_lastReturnValue instanceof BizletResponse)
        {
            value=((BizletResponse)m_lastReturnValue).getObject(type,name);
            if(value!=null)
                return value;
        }    
/*
        // first check the result values
        if(name.equals(&quot;result&quot;))
        {
            if(m_lastReturnValue==null)
                return null;
            if(m_lastReturnValue.getClass().isAssignableFrom(type))
            {
                return m_lastReturnValue;
            }
            throw new BizletException("Can't match type");
        }
*/
        if (parameters.containsKey(name))
        {
            Object obj = parameters.get(name);
            if (type.isInstance(obj))
                return obj;
            throw new BizletException("Can't match type");
        }

        //then check the getParameter methods
        <xsl:for-each select="Method" >
            <xsl:if test="starts-with(@Name,'get') and count(descendant::Parameter)=0"> <!-- eliminate any get method with parameters -->          
        if(name.equals(&quot;<xsl:value-of select="substring-after(@Name,'get')"/>&quot;))
        {
            Class c=(Class)parameters.get(&quot;<xsl:value-of select="substring-after(@Name,'get')"/>&quot;);
            if(c.isAssignableFrom(type))
            {
                return <xsl:value-of select="@Name"/>();
            }
            throw new BizletException("Can't match type");
        }
            </xsl:if>
        </xsl:for-each>
        return null;
    }
    
    public Iterator getParameterNames()  throws BizletException
    {
        //prepareParameterList();
        if((m_lastReturnValue!=null) &amp;&amp; (m_lastReturnValue instanceof BizletResponse))
        {
            return ((BizletResponse)m_lastReturnValue).getParameterNames();
        }    
        return parameters.keySet().iterator();                
    }

    final protected void prepareParameterList() throws BizletException
    {
        if(parameters==null)
        {    
            try{
                // building the parameter name list;
                parameters=new Hashtable();
                <xsl:for-each select="Method">
                    <xsl:if test="starts-with(@Name,'get')">
                parameters.put(&quot;<xsl:value-of select="substring-after(@Name,'get')"/> &quot;,Class.forName(&quot;<xsl:value-of select="./Signature/ReturnType[1]"/>&quot;));
                    </xsl:if>
                </xsl:for-each>
                //parameters.put(&quot;result&quot;,Class.forName("java.lang.Object"));
            }
            catch(Exception ex)
            {
                throw new BizletException("failed to construct parameter names list",ex);
            }
        }
    }    
    // End of Paremeter Accessor Section 
}
</xsl:template>

<!-- Start Individual Method invoker section -->
    <xsl:template match="Method" mode="methodInvoker">
        <xsl:variable name="MethodName" select="@Name"/>
    final private BizletResponse invoke_<xsl:value-of select="@Name"/>(ParameterAccessor accessor) throws ComergentException<xsl:text />
    {
        // defining the variables.                                         
        <xsl:call-template name="ParameterGetter">
            <!-- <xsl:with-param name="params" select="Signature/Parameter"/> -->
            <xsl:with-param name="params" select="Signature/Parameter[not(./@Name = ../preceding-sibling::Signature/Parameter/@Name)]"/>


        </xsl:call-template>                                    
        <!-- <xsl:apply-templates select="Signature" mode="methodInvoker"/> -->              
        // picking up the right signature to call
        // Pick the signature to call base on the longest matching algorithm.
        <xsl:for-each select="./Signature">
            <xsl:sort select="count(Parameter)" order="descending"/>
                <xsl:if test="count(Parameter)&gt;0">                
            if(<xsl:for-each select="./Parameter">(<xsl:value-of select="@Name"/>!=null)<xsl:if test="position()!=last()">&amp;&amp;</xsl:if></xsl:for-each>)
                </xsl:if>
            {
                <xsl:if test="./ReturnType='void'" >
                <xsl:value-of select="$MethodName"/>(<xsl:for-each select="Parameter"><xsl:value-of select="@Name"/><xsl:if test="position()!=last()">,</xsl:if></xsl:for-each>);
                m_lastReturnValue=null;    
                </xsl:if>
                <xsl:if test="./ReturnType!='void'" >
                m_lastReturnValue=<xsl:value-of select="$MethodName"/>(<xsl:for-each select="Parameter"><xsl:value-of select="@Name"/><xsl:if test="position()!=last()">,</xsl:if></xsl:for-each>);
                </xsl:if>
                return adjustReturnType(m_lastReturnValue, "<xsl:value-of select="./ReturnType"/>");
            }
                <xsl:if test="position()=last()">
                <xsl:if test="count(Parameter)&gt;0">
        // No match - can't invoke bizlet method
        throw new CannotInvokeBizletOperationException("<xsl:value-of select="@Name"/>");
                </xsl:if>
                </xsl:if>
        </xsl:for-each>

        <xsl:if test="count(./Signature)=0">
        // No match - can't invoke bizlet method
        throw new CannotInvokeBizletOperationException("<xsl:value-of select="@Name"/>");
        </xsl:if>                                                     
    }
    </xsl:template>
    
    <xsl:template name="ParameterGetter">
        <xsl:param name="params"/>
        <xsl:choose>
        <xsl:when test="$params"> 
            <xsl:variable name="first" select="$params[1]"/>         
                    
                    <xsl:variable name="methodName" select="name($first/../@Name)"/>        
                    <xsl:variable name="BizletName" select="name($first/../@Name)"/>                                  
                    <xsl:variable name="paramName" select="name($first/@Name)"/>        
                    <xsl:variable name="paramType" select="concat($first/@Type,'_get')"/>                 
       // ok ok
        <xsl:value-of select="$first/@Type"/><xsl:text> </xsl:text><xsl:value-of select="$first/@Name"/>;        
        try
        {
            <xsl:value-of select="$first/@Name"/>=(<xsl:value-of select="$first/@Type"/>)accessor.getObject(&quot;<xsl:value-of select="$first/@Type"/>&quot;, &quot;<xsl:value-of select="$first/@Name"/>&quot;);
        }
        catch(ConstraintViolationException cvex)
        {
             <xsl:value-of select="$first/@Name"/>=null;
        }                     
            <xsl:call-template name="ParameterGetter" >
                <xsl:with-param name="params" select="$params[position()!=1]"/>
            </xsl:call-template>
        </xsl:when>
        <xsl:otherwise/>
        </xsl:choose>               
    </xsl:template>
    
    <xsl:template match="Signature" mode="methodInvoker">        
        <xsl:variable name="methodName" select="name(../@Name)"/>        
        <xsl:variable name="BizletName" select="name(../@Name)"/>
            
        
        <xsl:for-each select="./Parameter" > -->
        <xsl:variable name="paramName" select="name(@Name)"/>        
        <xsl:variable name="paramType" select="concat(@Type,'_get')"/>                
       // ok ok        
        try
        {
            <xsl:value-of select="@Name"/>=(<xsl:value-of select="@Type"/>)accessor.getObject(&quot;<xsl:value-of select="@Type"/>&quot;, &quot;<xsl:value-of select="@Name"/>&quot;);
        }
        catch(ConstraintViolationException cvex)
        {
             <xsl:value-of select="@Name"/>=null;
        }        
        </xsl:for-each>                
    </xsl:template> 
<!-- End Individual Method invoker section -->

<!-- Start Validator Section -->
    <xsl:template match="Method" mode="methodParamValidator">
        <xsl:apply-templates mode="methodParamValidator"/>
        <xsl:variable name="methodName" select="@Name"/>
        <xsl:variable name="BizletName" select="../@Name"/>                          

        <xsl:for-each select="Signature/Parameter[not(./@Name = ../preceding-sibling::Signature/Parameter/@Name)]">
        <xsl:variable name="paramName" select="@Name"/>        
        <xsl:variable name="paramType"  select="@Type"/>
    // Methods used to validate each individual
    final public ConstraintViolation validate_<xsl:value-of select="$methodName"/>_<xsl:value-of select="$paramName"/>(<xsl:value-of select="$paramType" /><xsl:text> </xsl:text><xsl:value-of select="$paramName"/>)
    {
        // check for not null condition
        if (<xsl:value-of select="$paramName"/> == null)
        {
            return new NullConstraintViolation("<xsl:value-of select="$BizletName"/>", "<xsl:value-of select="$paramName"/>");
        }
// checking constraints
         <xsl:apply-templates select="Constraint">
            <xsl:with-param name="independent" select="5=5"/>
            <xsl:with-param name="name" select="$paramName"/>
            <xsl:with-param name="type" select="@Type"/>
            <xsl:with-param name="componentName" select="//JavaInterface/@Name"/>
        </xsl:apply-templates>
        return null;
    }    
    </xsl:for-each>
    </xsl:template>

    <xsl:template match="Signature" mode="methodParamValidator">
        <xsl:variable name="BizletName"  select="../@Name"/>
        <xsl:variable name="MethodName" select="../@Name"/>

    final public <xsl:value-of select="ReturnType"/><xsl:text> </xsl:text><xsl:value-of select="$MethodName"/>(<xsl:for-each select="Parameter" ><xsl:value-of select="@Type" /><xsl:text> </xsl:text><xsl:value-of select="@Name" /><xsl:if test="position()!=last()" ><xsl:text>, </xsl:text></xsl:if></xsl:for-each>) <xsl:if test="Throws"> throws <xsl:for-each select="Throws/Exception" > <xsl:value-of select="." /><xsl:if test="position()!=last()" ><xsl:text>, </xsl:text></xsl:if></xsl:for-each></xsl:if>
    {
        InputValues inputValues = null;
        boolean inputViolation = false;

        <xsl:apply-templates select="Parameter" mode="inline-validation"/>

        if (inputViolation)
        {
            // return the input params along with the violations
             <xsl:for-each select="./Parameter" >
                <xsl:variable name="paramName">
                    <xsl:value-of select="@Name"/>
                </xsl:variable>
            // save the input for parameters with no violations
            if (!inputValues.contains("<xsl:value-of select="$paramName"/>"))
                inputValues.addInputValue("<xsl:value-of select="$paramName"/>", new InputValue("<xsl:value-of select="$BizletName"/>","<xsl:value-of select="$paramName"/>", <xsl:value-of select="$paramName"/>));
            </xsl:for-each>
            throw new BizletValidationException(inputValues);
        }

        <xsl:if test="ReturnType='void'">
        bizlet.<xsl:value-of select="$MethodName"/>(<xsl:for-each select="Parameter" ><xsl:value-of select="@Name" /><xsl:if test="position()!=last()" ><xsl:text>, </xsl:text></xsl:if></xsl:for-each>);
        </xsl:if>

        <xsl:if test="ReturnType!='void'">
        return bizlet.<xsl:value-of select="$MethodName"/>(<xsl:for-each select="Parameter" ><xsl:value-of select="@Name" /><xsl:if test="position()!=last()" ><xsl:text>, </xsl:text></xsl:if></xsl:for-each>);
        </xsl:if>

    }
    </xsl:template>

<xsl:template match="Parameter" mode="inline-validation">
        <xsl:variable name="paramName" select="@Name"/>
        <xsl:variable name="BizletName" select="//JavaInterface/@Name"/>

        // check for not null condition
        if (<xsl:value-of select="$paramName"/> == null)
        {
            inputViolation = true;
            inputValues = addViolation(inputValues, new NullConstraintViolation("<xsl:value-of select="$BizletName"/>", "<xsl:value-of select="$paramName"/>"));
        }

        // checking constraints
         <xsl:apply-templates select="Constraint">
            <xsl:with-param name="name" select="$paramName"/>
            <xsl:with-param name="type" select="@Type"/>
            <xsl:with-param name="componentName" select="//JavaInterface/@Name"/>
        </xsl:apply-templates>
</xsl:template>


</xsl:stylesheet>

