<?xml version="1.0" ?>
<!--
  Licensed Materials - Property of IBM 

  IBM Sterling Selling and Fullfillment Suite 9.1 

  (c) Copyright IBM Corp 2001, 2010 All Rights Reserved

  US Government Users Restricted Rights - Use, duplication or disclosure
  restricted by GSA ADP Schedule Contract with IBM Corp.
-->
<xsl:stylesheet    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"   version="1.0">

<xsl:template name="getCustomizationStatus">
    <xsl:param name="class"/>
    <xsl:param name="classIsNew"/>
    <xsl:param name="outputType"/>

    <xsl:choose>
        <xsl:when test="not(contains($class/@moduleName,'.ref')) and not($classIsNew)">
            <xsl:choose>
                <xsl:when test="contains($class/@moduleName,'schema.api')">
                    <xsl:call-template name="writeCustomizationStatus">
                        <xsl:with-param name="status">red</xsl:with-param>
                        <xsl:with-param name="explanation">Modification of generated Data Beans</xsl:with-param>
                        <xsl:with-param name="outputType" select="$outputType"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="writeCustomizationStatus">
                        <xsl:with-param name="status">red</xsl:with-param>
                        <xsl:with-param name="explanation">Modification of non reference class</xsl:with-param>
                        <xsl:with-param name="outputType" select="$outputType"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:when>
        <xsl:when test="contains($class/@moduleName,'dcm') and $classIsNew">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">yellow</xsl:with-param>
                <xsl:with-param name="explanation">A creation of a new platform class</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
       <xsl:otherwise>
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">green</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:otherwise>
    </xsl:choose>

</xsl:template>


<xsl:template name="getSubclassStatus">
    <xsl:param name="class"/>
    <xsl:param name="classIsNew"/>
    <xsl:param name="baseClass"/>
    <xsl:param name="baseClassIsNew"/>
    <xsl:param name="outputType"/>

    <xsl:choose>
        <!-- Subclassing generated Data Beans -->
        <xsl:when test="contains($baseClass/@moduleName,'schema.api')">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">red</xsl:with-param>
                <xsl:with-param name="explanation">Generated data beans should not be subclassed</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <!-- Subclassing of internal classes -->
        <xsl:when test="contains($baseClass/@moduleName,'.impl') and not($baseClassIsNew)">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">red</xsl:with-param>
                <xsl:with-param name="explanation">Internal classes should not be subclassed</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <!-- APIs should not sub class implementation-->
        <xsl:when test="contains($class/@moduleName,'.api') and contains($baseClass/@moduleName,'.impl')">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">red</xsl:with-param>
                <xsl:with-param name="explanation">API classes should not sub class implementation</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <!-- APIs should not sub class reference-->
        <xsl:when test="contains($class/@moduleName,'.api') and contains($baseClass/@moduleName,'.ref')">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">red</xsl:with-param>
                <xsl:with-param name="explanation">API classes should not sub class reference</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <!-- Reference should not sub class implementation-->
        <xsl:when test="contains($class/@moduleName,'.ref') and contains($baseClass/@moduleName,'.impl')">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">red</xsl:with-param>
                <xsl:with-param name="explanation">API classes should not sub class implementation</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">green</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template name="getInterfaceImplStatus">
    <xsl:param name="class"/>
    <xsl:param name="classIsNew"/>
    <xsl:param name="interface"/>
    <xsl:param name="interfaceIsNew"/>
    <xsl:param name="outputType"/>

    <xsl:choose>
        <!-- implementing and internal interface -->
        <xsl:when test="contains($interface/@moduleName,'.impl') and not($interfaceIsNew)">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">red</xsl:with-param>
                <xsl:with-param name="explanation">Internal interfaces should not be implemented</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">green</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>


<xsl:template name="getReferenceStatus">
    <xsl:param name="class"/>
    <xsl:param name="classIsNew"/>
    <xsl:param name="refClass" />
    <xsl:param name="refClassIsNew"/>
    <xsl:param name="outputType"/>

    <xsl:choose>

        <!-- Check for reference to internal classes -->
        <xsl:when test=" ($classIsNew) and not($refClassIsNew) and contains($refClass/@moduleName,'.impl')">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">yellow</xsl:with-param>
                <xsl:with-param name="explanation">Project classes should not reference internal Comergent Classes</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <!-- API or reference should not reference implementation classes-->
        <xsl:when test=" ($classIsNew) and (contains($class/@moduleName,'.api') or contains($class/@moduleName,'.ref')) and contains($refClass/@moduleName,'.impl')">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">yellow</xsl:with-param>
                <xsl:with-param name="explanation">API or reference should not reference implementation classes</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <!-- Implementation of one module cannot call an implementation of another module-->
        <xsl:when test=" ($classIsNew) and contains($class/@moduleName,'.impl') and contains($refClass/@moduleName,'.impl') and ($class/@moduleName!=$refClass/@moduleName)">
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">yellow</xsl:with-param>
                <xsl:with-param name="explanation">API or reference should not reference implementation classes</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
            <xsl:call-template name="writeCustomizationStatus">
                <xsl:with-param name="status">green</xsl:with-param>
                <xsl:with-param name="outputType" select="$outputType"/>
            </xsl:call-template>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>
</xsl:stylesheet>

